###############################################################################
#
# Copyright (C) 2013 Wind River Systems, Inc.
#
# This code is licensed under the GPLv2.
#
###############################################################################
import sys
import string
import os
import random
import re
import ovp_ott_executor as EXEC
import ovp_ott_analysis_result as AR


###############################################################################
# Class definition
#
# This class is responsible for comparing the /proc/config.gz file contents
# (the kernel configuration parameters) with some tuning configuration
# parameters, defined in various tuning profiles.
###############################################################################
class OVP_OTT_KERNEL_ANALYSER:
    # this file is a must have for the kernel analyser.
    kernelConfigFile = "/proc/config.gz"

    # Class initializer; if not specified the kernel configuration file
    # name is initialized here
    def __init(self):
        if OVP_OTT_KERNEL_ANALYSER.kernelConfigFile == None:
            OVP_OTT_KERNEL_ANALYSER.kernelConfigFile = "/proc/config.gz"


    # Reads the kernel configuration file contents into an array of strings.
    def readKernelConfig(self):
        # create a random named file
        char_set = string.ascii_uppercase + string.digits
        tmpFileName = "/tmp/" + ''.join(random.sample(char_set*6,6))
        # decompress the kernel configuration into a temp file
        cmd = "zcat " + OVP_OTT_KERNEL_ANALYSER.kernelConfigFile
        cmd = cmd + " > " + tmpFileName
        executor = EXEC.OVP_OTT_EXECUTOR()
        executor.executeCmd(cmd)
        # read the tempfile into the memory
        with open(tmpFileName) as kFile:
            kernelConfigLines = kFile.read().splitlines()
            kFile.close()

        # delete the temporary file
        os.remove(tmpFileName)
        # return the lines
        return kernelConfigLines


    # Reads the pFileName specified file content into an array of strings.
    def readFileIntoMemory(self, pFileName):
        lines = []
        try:
            with open(pFileName) as pf:
                lines = pf.read().splitlines()
        except IOError:
            sys.stderr.write("Error occured processing file: " + pFileName)

        # eliminate the empty lines
        lines = filter(None, lines)
        # keep the lines which contains '=y'
        lines1 = filter(lambda k: '=y' in k, lines)
        # keep the lines which contains 'is not set'
        lines2 = filter(lambda k: 'is not set' in k, lines)
        # sum-up to be grouped...
        lines = lines1 + lines2
        return lines


    # Compare two array of strings:
    # pLinesToCompare contains the lines from the tuned configuration, and
    # these values shall be looked up into the global kernel configuration
    # file, which content is in the pKernelLines parameter.
    # The pResults parameter is an OVP_OTT_ANALYSIS_RESULT object instance,
    # into which one, a pResultTableName table (array) will be created, which
    # will hold the comparison results (tuples, as described in the result
    # class).
    def compareWithKernelLines(self, pLinesToCompare, pKernelLines,
                               pResults, pResultTableName):
        # add a new result list with the given name....
        pResults.createNewResultList(pResultTableName)

        for lineToCompare in pLinesToCompare:
            if lineToCompare not in pKernelLines:
                lookupField = ""
                match = re.match("^#\s*(.*?)\s+(.*)$", lineToCompare)
                if match:
                    lookupField = match.group(1)
                else:
                    match = re.match("^(.*)=(.*)$", lineToCompare)
                    if match:
                        lookupField = match.group(1)

                # in lookupField we have the kernel parameter which differs.
                # we know his value from the initial file, but not in the
                # kernel config file... so, look-it-up!
                lookupString = "^(.*)" + lookupField + "(.*)$"
                resultToReport = (lookupField, "", lineToCompare)
                for kernelLine in pKernelLines:
                    match = re.match(lookupString, kernelLine)
                    if match:
                        resultToReport = (lookupField, kernelLine, lineToCompare)
                        break
                # add the results to the result list
                pResults.addResultToList(pResultTableName, resultToReport)
