###############################################################################
#
# Copyright (C) 2013 Wind River Systems, Inc.
#
# This code is licensed under the GPLv2.
#
###############################################################################
import sys
import string
import ovp_ott_abs_reporter as ABSR


###############################################################################
# Class definition
#
# This class extends (implements) the ovp_ott_abs_reporter class, and is
# responsible for creating the final report in HTML format.
###############################################################################
class OVP_OTT_HTML_REPORTER(ABSR.OVP_OTT_ABS_REPORTER):

    # The main access point to this class; implements the report logic
    # and calls all the helper methods.
    def writeReport(self, pOvpOttAnalysisResult):
        self.result = pOvpOttAnalysisResult
        self.totalGroups = 0
        self.separatorLine="<hr/>"
        self.fileName = self.getReportName() + ".html"
        self.outFile = None

        try:
            self.outFile = open(self.fileName, "w")
        except IOError:
            sys.stderr.write("Error creating file: " + self.fileName)

        self.writeReportHeader()
        self.writeReportSummary()
        for gr in range(0, self.totalGroups):
            self.writeReportGroup(gr)
        self.writeReportFooter()

        if self.outFile:
            self.outFile.close()


    # "Abstractization" of write... It's used to ease the write method
    # implementation handling.
    # In the case when the main entry point to the class was able to create a
    # result file, all the prints will go to that file, otherwise it will try
    # to print to the console.
    def writeOut(self, pStr):
        if self.outFile:
            self.outFile.write(pStr)
        else:
            print pStr


    # Will enclose the given parameter string between the HTML <center> tags
    # and will call the class write method
    def writeCenteredString(self, pString):
        stri = "<center>" + pString + "</center>"
        self.writeOut(stri)


    # Creates a new string which will be guarded by pTag and pEndTag.
    # If the pEndTag parameter is not defined, then the method will
    # automatically calculate the end-tag from the pTag parameter.
    # If both pTag and pEndTag parameters aren't defined, then the method will
    # return the pString input parameter as result.
    def createTaggedString(self, pString, pTag=None, pEndTag=None):
        stri = ""
        if pTag != None:
            stri = "<" + pTag + ">"
        stri = stri + pString
        if pEndTag == None:
            stri = stri + "</" + pTag + ">"
        else:
            stri = stri + "</" + pEndTag + ">"
        stri = stri + "\n"
        return stri


    # This method will create a tagged string (by calling the associated
    # class method), and will call the class's write method.
    def writeTaggedString(self, pString, pTag=None, pEndTag=None):
        stri = self.createTaggedString(pString, pTag, pEndTag)
        self.writeOut(stri)


    # This method implements the HTML document header. It is responsible to
    # correctly indicate the document type, and create the HTML document
    # header stuff
    def writeDocumentHeader(self):
        hStr = "<!DOCTYPE html PUBLIC '-//W3C//DTD HTML 3.2//EN'>"
        hStr += "<html>\n\t<head>\n\t\t<title>"
        hStr += self.getReportName()
        hStr += "</title>\n\t</head>\n\n<body>"
        self.writeOut(hStr)


    # This method writes the HTML document footer, and is responsible to
    # correctly close the document body and html tags.
    def writeDocumentFooter(self):
        fStr = "</body>\n</html>"
        self.writeOut(fStr)


    # This method writes out the report header (title and date) information.
    def writeReportHeader(self):
        self.writeDocumentHeader()
        topStr = "\n<a id='top'></a>\n"
        self.writeOut(topStr)
        self.writeOut(self.separatorLine)
        self.writeOut("<center>")
        self.writeTaggedString(self.getReportName(), "h1")
        self.writeTaggedString(self.getReportDate(), "h4")
        self.writeOut("</center>")
        self.writeOut(self.separatorLine)


    # This method writes out the report footer information.
    def writeReportFooter(self):
        self.writeOut(self.separatorLine)
        fStr = "<center><font size=2> \
        Generated by OVP_OTT \
        </font></center>"
        self.writeOut(fStr)
        self.writeOut(self.separatorLine)
        self.writeDocumentFooter()


    # Writes in a tabular form a small summary about the report. the number of
    # groups (list of results), and the total number of analysed parameters.
    def writeReportSummary(self):
        self.writeOut("<center>")
        self.writeTaggedString("Summary", "h2")
        self.totalGroups = self.result.getNumberOfLists()
        totNum = self.result.getTotalNumberOfElements()
        tableStr = "<table align='center' border='1' width='50%' \
        summary='summary table'>\n"
        # total number of groups
        tableStr += "<tr>\n"

        tableStr += "\t<td>"
        tableStr += "Total report groups"
        tableStr += "</td>\n"

        tableStr += "\t<td>"
        tableStr += str(self.totalGroups)
        tableStr += "</td>\n"

        tableStr += "</tr>\n"

        # total number of elements
        tableStr += "<tr>\n"

        tableStr += "\t<td>"
        tableStr += "Overall Total Elements"
        tableStr += "</td>\n"

        tableStr += "\t<td>"
        tableStr += str(totNum)
        tableStr += "</td>\n"

        tableStr += "</tr>\n"

        tableStr += "</table>\n\n"
        self.writeOut(tableStr)

        # after the table, write a hyperlink line to each section found...
        for gr in range(0, self.totalGroups):
            grTitle = self.result.getResultGroupName(gr)
            grRef = "\n<a href='#" + grTitle + "'>" + grTitle + "</a>\n"
            self.writeOut(grRef)

        self.writeOut("</center>")


    # This method writes out in a tabular form the result data. Thus, for each
    # result list in the results, it will create a new table. The table columns
    # are in concordance with the data in the result list elements (tuples)
    def writeReportGroup(self, pGroupNumber):
        grTitle = self.result.getResultGroupName(pGroupNumber)
        self.writeOut(self.separatorLine)
        grRef = "\n<a id='" + grTitle + "'></a>\n"
        self.writeOut(grRef)
        grTitle = self.createTaggedString(grTitle, "h2")
        self.writeTaggedString(grTitle, "center")
        topRef = "\n<center><a href='#top'>~ top ~</a></center>\n"
        self.writeOut(topRef)
        self.writeOut(self.separatorLine)
        group = self.result.getResultGroup(pGroupNumber)
        tabHead = "<center><table border='1' width='90%' align='center' \
        summary='Report table'>"
        tabHead += "<tr>"
        tabHead += self.createTaggedString("Parameter", "th")
        tabHead += self.createTaggedString("Current Value", "th")
        tabHead += self.createTaggedString("Desired Value", "th")
        tabHead += "</tr>"
        self.writeOut(tabHead)

        for elem in group:
            line = "<tr>"
            line += self.createTaggedString(elem[0], "td")
            line += self.createTaggedString(elem[1], "td")
            line += self.createTaggedString(elem[2], "td")
            line += "</tr>\n"
            self.writeOut(line)
        tblEnd = "</table></center>\n"
        self.writeOut(tblEnd)
